==> la_catch-0.0.4-py3-none-any.whl: la_catch/__init__.py <==
from asyncio import iscoroutinefunction
from inspect import Signature, signature
from typing import Any, Awaitable, Callable


class Catch:
    def __init__(
        self,
        exceptions: Exception | tuple[Exception] = Exception,
        callback: Callable | Awaitable | Any = None,
        *args,
        **kwargs,
    ):
        if not isinstance(exceptions, tuple):
            exceptions = (exceptions,)

        self._exceptions = exceptions
        self._callback = callback
        self._args = args
        self._kwargs = kwargs

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback) -> bool:
        if type not in self._exceptions:
            return False

        self._kwargs |= {"exception": value}

        if callable(self._callback):
            self._callback(*self._args, **self._kwargs)

        return True

    def __call__(self, func: Callable) -> Callable:
        def wrapper(*args, **kwargs):
            args, kwargs = self._get_arguments(func, args, kwargs)

            try:
                return func(*args, **kwargs)
            except self._exceptions as e:
                if callable(self._callback):
                    kwargs |= {"exception": e}
                    return self._callback(*args, **kwargs)
                return self._callback

        async def awrapper(*args, **kwargs):
            args, kwargs = self._get_arguments(func, args, kwargs)

            try:
                return await func(*args, **kwargs)
            except self._exceptions as e:
                if iscoroutinefunction(self._callback):
                    kwargs |= {"exception": e}
                    return await self._callback(*args, **kwargs)
                elif callable(self._callback):
                    kwargs |= {"exception": e}
                    return self._callback(*args, **kwargs)
                return self._callback

        if iscoroutinefunction(func):
            return awrapper
        return wrapper

    async def __aenter__(self):
        return self

    async def __aexit__(self, type, value, traceback) -> bool:
        if type not in self._exceptions:
            return False

        self._kwargs |= {"exception": value}

        if callable(self._callback) and iscoroutinefunction(self._callback):
            await self._callback(*self._args, **self._kwargs)
        elif callable(self._callback):
            self._callback(*self._args, **self._kwargs)

        return True

    def _get_arguments(
        self, func: Callable, args: tuple, kwargs: dict
    ) -> tuple[tuple, dict]:
        """Get function arguments, including default values"""

        sign = signature(func)
        bound_arguments = sign.bind(*args, **kwargs)
        bound_arguments.apply_defaults()
        args = bound_arguments.args
        kwargs = bound_arguments.kwargs

        return args, kwargs

==> la_catch-0.0.5-py3-none-any.whl: la_catch/__init__.py <==
from asyncio import iscoroutinefunction
from typing import Any, Awaitable, Callable

from la_catch.utility import get_arguments


class Catch:
    def __init__(
        self,
        exceptions: Exception | tuple[Exception] = Exception,
        callback: Callable | Awaitable | Any = None,
        *args,
        **kwargs,
    ):
        if not isinstance(exceptions, tuple):
            exceptions = (exceptions,)

        self._exceptions = exceptions
        self._callback = callback
        self._args = args
        self._kwargs = kwargs

        self.callback_return = None

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback) -> bool:
        if type not in self._exceptions:
            return False

        self._kwargs |= {"exception": value}

        if callable(self._callback):
            self.callback_return = self._callback(*self._args, **self._kwargs)
        else:
            self.callback_return = self._callback

        return True

    def __call__(self, func: Callable) -> Callable:
        def wrapper(*args, **kwargs):
            args, kwargs = get_arguments(func=func, args=args, kwargs=kwargs)

            with self.__class__(
                self._exceptions, self._callback, *args, **kwargs
            ) as catch:
                return func(*args, **kwargs)
            return catch.callback_return

        async def awrapper(*args, **kwargs):
            args, kwargs = get_arguments(func=func, args=args, kwargs=kwargs)

            async with self.__class__(
                self._exceptions, self._callback, *args, **kwargs
            ) as catch:
                return await func(*args, **kwargs)
            return catch.callback_return

        if iscoroutinefunction(func):
            return awrapper
        return wrapper

    async def __aenter__(self):
        return self

    async def __aexit__(self, type, value, traceback) -> bool:
        if type not in self._exceptions:
            return False

        self._kwargs |= {"exception": value}

        if callable(self._callback) and iscoroutinefunction(self._callback):
            self.callback_return = await self._callback(*self._args, **self._kwargs)
        elif callable(self._callback):
            self.callback_return = self._callback(*self._args, **self._kwargs)

        return True
